@@ -96,10 +96,14 @@ setClass("idenseMatrix", representation("VIRTUAL"),
 setClass("ndenseMatrix", representation(x = "logical", "VIRTUAL"),
 	 contains = c("nMatrix", "denseMatrix"))
 
+## virtual SPARSE ------------
+
+setClass("sparseMatrix", representation("VIRTUAL"), contains = "Matrix")
+
 ## diagonal: has 'diag' slot;  diag = "U"  <--> have identity matrix
 setClass("diagonalMatrix", representation(diag = "character", "VIRTUAL"),
-	 contains = "denseMatrix",
-	 ## TODO??   ^^^^^ extend just "Matrix" !?!?!? -- against DSC'07 talk (3D classes)
+	 contains = "sparseMatrix",
+         ## NOTE:    ^^^^^^ was dense Matrix, until 0.999375-11 (2008-07)
 	 validity = function(object) {
 	     d <- object@Dim
 	     if(d[1] != (n <- d[2])) return("matrix is not square")
@@ -113,10 +117,6 @@ setClass("diagonalMatrix", representation(diag = "character", "VIRTUAL"),
 	 prototype = prototype(diag = "N")
 	 )
 
-## virtual SPARSE ------------
-
-setClass("sparseMatrix", representation("VIRTUAL"), contains = "Matrix")
-
 ## sparse matrices in Triplet representation (dgT, lgT, ..):
 setClass("TsparseMatrix", representation(i = "integer", j = "integer", "VIRTUAL"),
 	 contains = "sparseMatrix",
@@ -150,6 +150,17 @@ setClass("isparseMatrix", representation("VIRTUAL"),
 setClass("nsparseMatrix", representation("VIRTUAL"),
 	 contains = c("nMatrix", "sparseMatrix"))
 
+if(FALSE) { ##-- a nice idea --- but needs more careful method re-definitions
+            ##-- such that the *correct* methods are dispatched:
+## Trying to use more economical method defs:
+setClass("dCsparseMatrix", representation("VIRTUAL"),
+	 contains = c("CsparseMatrix", "dsparseMatrix"))
+setClass("lCsparseMatrix", representation("VIRTUAL"),
+	 contains = c("CsparseMatrix", "lsparseMatrix"))
+setClass("nCsparseMatrix", representation("VIRTUAL"),
+	 contains = c("CsparseMatrix", "nsparseMatrix"))
+}
+
 ## ------------------ Proper (non-virtual) Classes ----------------------------
 
 ##----------------------  DENSE	 -----------------------------------------
@@ -326,9 +337,9 @@ setClass("dsCMatrix",
 	 validity = function(object) .Call(tCMatrix_validate, object)
 	 )
 
-if(FALSE) ## FIXME ??? Class of positive definite (Csparse symmetric) Matrices:
+if(FALSE) ## TODO ??? Class of positive definite (Csparse symmetric) Matrices:
 setClass("dpCMatrix", contains = "dsCMatrix",
-	 validity = function(object) FIXME("test for pos.definite ??"))
+	 validity = function(object) TODO("test for pos.definite ??"))
 
 ## numeric, sparse, sorted compressed sparse row-oriented general matrices
 setClass("dgRMatrix",
@@ -430,7 +441,12 @@ setClass("nsTMatrix",
          ## validity: Tsparse_ and symmetric*_validate should be enough
 	 )
 
-## nonzero pattern, sparse, sorted compressed column-oriented general matrices
+## nonzero pattern, sparse, sorted compressed column-oriented matrices
+setClass("ngCMatrix",
+	 contains = c("CsparseMatrix", "nsparseMatrix", "generalMatrix"),
+         ## validity: Csparse_validate should be enough
+	 )
+
 setClass("ngCMatrix",
 	 contains = c("CsparseMatrix", "nsparseMatrix", "generalMatrix"),
          ## validity: Csparse_validate should be enough
@@ -544,13 +560,16 @@ setClass("pMatrix", representation(perm = "integer"),
 ## Mother class:
 setClass("MatrixFactorization", representation(Dim = "integer", "VIRTUAL"))
 
+setClass("CholeskyFactorization", representation = "VIRTUAL",
+         contains = "MatrixFactorization")
+
 ## -- Those (exceptions) inheriting from "Matrix" : ---
 
-setClass("Cholesky",  contains = c("dtrMatrix", "MatrixFactorization"))
+setClass("Cholesky",  contains = c("dtrMatrix", "CholeskyFactorization"))
 
-#unUsed: setClass("LDL", contains = c("dtrMatrix", "MatrixFactorization"))
+#unUsed: setClass("LDL", contains = c("dtrMatrix", "CholeskyFactorization"))
 
-setClass("pCholesky", contains = c("dtpMatrix", "MatrixFactorization"))
+setClass("pCholesky", contains = c("dtpMatrix", "CholeskyFactorization"))
 
 ## These are currently only produced implicitly from *solve()
 setClass("BunchKaufman",
@@ -569,7 +588,7 @@ setClass("pBunchKaufman",
 ## -- the usual ``non-Matrix'' factorizations : ---------
 
 setClass("CHMfactor",		 # cholmod_factor struct as S4 object
-	 contains = "MatrixFactorization",
+	 contains = "CholeskyFactorization",
 	 representation(colcount = "integer", perm = "integer",
 			type = "integer", "VIRTUAL"),
 	 validity = function(object) .Call(CHMfactor_validate, object))
@@ -612,6 +631,8 @@ setClass("sparseQR", contains = "MatrixFactorization",
 	 representation(V = "dgCMatrix", beta = "numeric",
 			p = "integer", R = "dgCMatrix", q = "integer"))
 
+##-- "SPQR" ---> ./spqr.R  for noew
+
 ## "denseQR" -- ?  (``a version of''  S3 class "qr")
 
 if (FALSE) { ## unused classes
@@ -664,10 +685,77 @@ setClass("Schur", contains = "MatrixFactorization",
 
 ## --- "General" (not Matrix at all) ----
 
+## e.g. for "Arith" methods:
+setClassUnion("numLike", members = c("numeric", "logical"))
+
+setClassUnion("numIndex", members = "numeric")
+
+## Note "rle" is a sealed oldClass (and "virtual" as w/o prototype)
+setClass("rleDiff", representation(first = "numLike", rle = "rle"),
+	 prototype = prototype(first = integer(),
+		     ## workaround buglet in R <= 2.10.x: rle = rle(integer())
+	 rle = structure(list(lengths = integer(0L), values = 0[0L]), class = "rle")
+	 ),
+	 validity = function(object) {
+	     if(length(object@first) != 1)
+		 return("'first' must be of length one")
+	     rl <- object@rle
+	     if(!is.list(rl) || length(rl) != 2 ||
+		!identical(sort(names(rl)), c("lengths", "values")))
+		 return("'rle' must be a list (lengths = *, values = *)")
+	     if(length(lens <- rl$lengths) != length(vals <- rl$values))
+		 return("'lengths' and 'values' differ in length")
+	     if(any(lens <= 0))
+		 return("'lengths' must be positive")
+	     TRUE
+	 })
+
+### 2010-03-04 -- thinking about *implenting* some 'abIndex' methodology,
+### I conclude that the following structure would probably be even more
+### efficient than the "rleDiff" one :
+### for now, at least use it, and define  "seqMat" <--> "abIndex" coercions:
+setClass("seqMat", contains = "matrix",
+	 prototype = prototype(matrix(0, nrow = 2, ncol=0)),
+	 validity = function(object) {
+	     if(!is.numeric(object)) return("is not numeric")
+	     d <- dim(object)
+	     if(length(d) != 2 || d[1] != 2)
+		 return("not a	 2 x n	matrix")
+	     if(any(object != floor(object)))
+		 return("some entries are not integer valued")
+	     TRUE
+	 })
+
+setClass("abIndex", # 'ABSTRact Index'
+         representation(kind = "character",
+                        ## one of ("int32", "double", "rleDiff")
+                                        # i.e., numeric or "rleDiff"
+                        x = "numLike", # for  numeric [length 0 otherwise]
+                        rleD = "rleDiff"),  # "rleDiff" result
+         prototype = prototype(kind = "int32", x = integer(0)),# rleD = ... etc
+         validity = function(object) {
+            switch(object@kind,
+                   "int32" = if(!is.integer(object@x))
+                   return("'x' slot must be integer when kind is 'int32'")
+                   ,
+                   "double" = if(!is.double(object@x))
+                   return("'x' slot must be double when kind is 'double'")
+                   ,
+                   "rleDiff" = {
+                       if(length(object@x))
+                   return("'x' slot must be empty when kind is 'rleDiff'")
+                   },
+                   ## otherwise
+                   return("'kind' must be one of (\"int32\", \"double\", \"rleDiff\")")
+                   )
+            TRUE
+         })
+
 ## for 'i' in x[i] or A[i,] : (numeric = {double, integer})
+## TODO: allow "abIndex" as well !
 setClassUnion("index", members =  c("numeric", "logical", "character"))
 
-## "atomic vectors" (-> ?is.atomic ) --
+## "atomic vectors" (-> ?is.atomic ) -- but note that is.atomic(<matrix>) !
 ## ---------------  those that we want to convert from old-style "matrix"
 setClassUnion("atomicVector", ## "double" is not needed, and not liked by some
 	      members = c("logical", "integer", "numeric",
@@ -675,23 +763,33 @@ setClassUnion("atomicVector", ## "double" is not needed, and not liked by some
 
 ## --- Matrix - related (but not "Matrix" nor "Decomposition/Factorization):
 
-### for 'value' in  x[..] <- value hence for all "contents" of our Matrices:
+## for 'value' in  x[..] <- value hence for all "contents" of our Matrices:
 setClassUnion("replValue", members = c("numeric", "logical", "complex", "raw"))
 
 ### Sparse Vectors ---- here use 1-based indexing ! -----------
+
+## 'longindex' should allow sparseVectors of "length" > 2^32,
+## which is necessary e.g. when converted from large sparse matrices
+## setClass("longindex", contains = "numeric")
+## but we use "numeric" instead, for simplicity (efficiency?)
 setClass("sparseVector",
-         representation(length = "integer", i = "integer", "VIRTUAL"),
+         representation(length = "numeric", i = "numeric", "VIRTUAL"),
+         ##                     "longindex"    "longindex"
+         ## note that "numeric" contains "integer" (if I like it or not..)
+	 prototype = prototype(length = 0),
          validity = function(object) {
              n <- object@length
-             if(any(object@i < 1L) || any(object@i > n))
+	     if(any(!is.finite(i <- object@i)))# no NA's !
+		 sprintf("'i' slot is not all finite")
+	     else if(any(i < 1) || any(i > n))
                  sprintf("'i' must be in 1:%d", n)
              else TRUE
          })
 .validXspVec <- function(object) {
-	     n <- object@length
-	     if(length(object@i) != length(object@x))
-		 "'i' and 'x' differ in length"
-	     else TRUE
+    ## n <- object@length
+    if(length(object@i) != length(object@x))
+        "'i' and 'x' differ in length"
+    else TRUE
 }
 setClass("dsparseVector",
 	 representation(x = "numeric"), contains = "sparseVector",
@@ -720,4 +818,3 @@ setClass("determinant",
 			logarithm = "logical",
 			sign = "integer",
 			call = "call"))
-
